﻿using System;
using System.ComponentModel;
using SLCommandingDemo.Commanding;

namespace SLCommandingDemo
{
    public class WordViewModel : INotifyPropertyChanged
    {
        #region Fields

        readonly Word _word;

        #endregion // Fields

        #region Constructor

        public WordViewModel(Word word)
        {
            _word = word;
        }

        #endregion // Constructor

        #region CapitalizeCommand

        RelayCommand _capitalizeCommand;

        public RelayCommand CapitalizeCommand
        {
            get
            {
                if (_capitalizeCommand == null)
                    _capitalizeCommand = new RelayCommand(
                        () => this.Capitalize(),
                        () => this.CanCapitalize);

                return _capitalizeCommand;
            }
        }

        void Capitalize()
        {
            this.Value = this.Value.ToUpper();
        }

        bool CanCapitalize
        {
            get { return !String.IsNullOrEmpty(this.Value) && this.Value != this.Value.ToUpper(); }
        }

        #endregion // CapitalizeCommand

        #region RemoveSubstringCommand

        RelayCommand<int> _removeSubstringCommand;

        public RelayCommand<int> RemoveSubstringCommand
        {
            get
            {
                if (_removeSubstringCommand == null)
                    _removeSubstringCommand = new RelayCommand<int>(
                        parameter => this.RemoveSubstring(parameter),
                        parameter => this.CanRemoveSubstring(parameter));

                return _removeSubstringCommand;
            }
        }

        void RemoveSubstring(int startIndex)
        {
            if (this.Value.Length < startIndex)
                this.Value = String.Empty;
            else
                this.Value = this.Value.Substring(startIndex);
        }

        bool CanRemoveSubstring(int startIndex)
        {
            return !String.IsNullOrEmpty(this.Value);
        }

        #endregion // RemoveSubstringCommand

        #region Value

        public string Value
        {
            get { return _word.Value; }
            set
            {
                if (value == _word.Value)
                    return;

                _word.Value = value;

                this.OnPropertyChanged("Value");

                this.CapitalizeCommand.RaiseCanExecuteChanged();
                this.RemoveSubstringCommand.RaiseCanExecuteChanged();
            }
        }

        #endregion // Value

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = this.PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion // INotifyPropertyChanged Members
    }
}